#!/usr/bin/perl -w
#
# AUTHOR: Eric W. Sarjeant <eric@sarjeant.com>
#         http://micromux.com
#
# DESCRIPTION
# Script to link iTunes media to a folder used to publish movies or TV shows to a TiVo.
# Meta-data is extracted from the iTunes media to create TiVo parsable text files. To
# provide multimedia publishing services to a TiVo you may want to look at pyTivo
# <http://pytivo.sourceforge.net/>.
#
# Links are symbolic and are created as a hard link to the iTunes media files. This allows
# the movie files to be browsable via a remote share with cover art visible.
#
# Requires AtomicParsley to work, basic install steps:
#
# 1. Install AtomicParsley.
#      sudo apt-get install atomicparsley
#
# 2. Install AtomicParsley::Command for Perl
#      sudo perl -MCPAN -e 'install AtomicParsley::Command' 
#
#
use strict;
use warnings;
use threads;

use File::Find::Rule;
use File::Basename;
use AtomicParsley::Command;
use Linux::Inotify2;

my $destPath;

sub linkMovieByGenre
{

    my($filepath, $destpath) = @_;

    # skip hidden directories...
    if ( $filepath !~ /\/\./ )
    {
        my $ap = AtomicParsley::Command->new({
	    ap => '/usr/bin/AtomicParsley',
	    verbose => 0,
        });
    
        # read tags from file
        my $tags = $ap->read_tags($filepath);
        my $genre = $tags->{"genre"};

        if (defined $genre and length($genre) > 0)
        {
            #print "   process file: $filepath...$destpath". $genre . "\n";
            my ($name, $path, $suffix) = fileparse($filepath);

            my $destdir = $destpath . $genre . "/";
            my $destfile = $destdir . $name;

            # make sure destination directory exists
            unless ( -e $destdir )
            {
                mkdir $destdir;
                mkdir $destdir . "/.meta";
            }

            # put the movies into place...
            unless ( -e $destfile )
            {
                print "adding $name to $destdir\n";
                link($filepath, $destfile);
                chmod oct("0644"), $destfile;
            }

            # put TiVo meta files into place
            my $tivoMetaFile = $destdir . ".meta/" . $name . ".txt";
            my $hasChanges = 0;

            # if a meta-file exists then check it for updating
            # otherwise we will need to create one if not already there
            if ( -f $tivoMetaFile )
            { 

                my $mtime = (stat($tivoMetaFile))[9];
                my $vtime = (stat($filepath))[9];

                if ($vtime > $mtime)
                {
                    $hasChanges++;
                }
            
            }
            else
            {
                $hasChanges = 1;
            }

            # check for changes or a non-existent file
            if ($hasChanges > 0)
            {
                writeTivoMetaData($tags, $tivoMetaFile);
            }
        }
    }

}

sub writeTivoMetaData
{
    my ($tags, $tivoMetaFile) = @_;

    print ("Create TiVo Meta File: " . $tivoMetaFile . "\n");

    open FILE, ">$tivoMetaFile" or die $!;
    my $title = $tags->{"title"};
    my $year = $tags->{"year"};
    my $desc = $tags->{"description"};
    my $genre = $tags->{"genre"};

    if (defined $title) 
    {
      print FILE "title : " . $title ."\n";
    }

    if (defined $year)
    {
        print FILE "movieYear : " . substr($year, 0, 4) . "\n";
    }

    if (defined $desc)
    {
        print FILE "description : " . $desc . "\n";
    }

    if (defined $genre)
    {
        print FILE "vProgramGenre : " . $genre . "\n";
    }

    close FILE;

}

sub recurseMovieFolder
{
    my ($path) = @_;

    print "Check Movie Path: $path\n";

    my $videos = File::Find::Rule->file->name("*.m4v")->start($path);
    while (defined (my $movie = $videos->match))
    {
        linkMovieByGenre($movie, $destPath);
    }

}

sub watchMovieFolder
{
    my ($srcPath) = @_;

    my $notify = new Linux::Inotify2 or die "Unable to monitor directory.\n";

    $notify->watch($srcPath, IN_CREATE, sub
        {
          my $e = shift;
          my $name = $e->fullname;

          #start watching for the M4V
          threads->create("startMovieFileWatch", $name);
        });
    
    1 while $notify->poll;

}

sub startMovieFileWatch
{

    my ($moviePath) = @_;
    print "startMovieFileWatch: $moviePath\n";

    my $movieNotify = new Linux::Inotify2;
    my $waitcount = 0;

    $movieNotify->watch($moviePath, IN_MOVED_TO, sub
        {
          my $e = shift;
          my $name = $e->fullname;
          print ("FOUND MOVIE: $name -- $destPath\n");
          linkMovieByGenre($name, $destPath);

          $e->w->cancel;
          $waitcount++;
        });

    while (0 == $waitcount)
    {
      threads->yield();
      $movieNotify->poll;
    }
}

#
# Application Startup
#
# Pass command line arguments or display script usage
#
if ($#ARGV > 0)
{

    unless ( -d $ARGV[1] )
    {
        die "iTunes Path Not Found: " . $ARGV[0] . "\n\n";
    }

    unless ( -d $ARGV[2] )
    {
        die "TiVo Path Not Found: " . $ARGV[1] . "\n\n";
    }
    else
    {
        $destPath = $ARGV[2];
    }

    if ( $ARGV[0] eq "batch" )
    {
        recurseMovieFolder($ARGV[1]);
    }
    else
    {
        watchMovieFolder($ARGV[1]);
    }
}
else
{

    print "\nAtomicSage\n\n";
    print "Clone iTunes media (movies / TV shows) to a folder being published to a TiVo.";
    print "Require pyTivo or some other mechanism to present your directory as available ";
    print "for a TiVo machine on your LAN.\n\n";

    print "    Usage:  AtomicSage.pl [batch | monitor] [iTunes Path] [TiVo Path]\n\n";

}

exit 0;
